;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;                                                                                        ;
; JonOS [Genesis] 0.0.1                                                                  ;
;  Made by GigaWolf (GigaWolf@Hotmail.com)                                               ;
;   Released under the GNU General Public License, Version 2                             ;
;                                                                                        ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

%define BOOT_SEG	0x7C0
%define DATA_SEG	0x1000
%define CODE_SEG	0x1000
%define STACK_SEG	0x3000
%define STACK_BOTTOM	0xFFFF
%define SECTORS		0xF
%define SIGNATURE	0x55AA
;========================================================================================

[BITS 16]
[ORG 0]

;Switch from 0000:7c00 to 07c0:0000
;----------------------------------------------------------------------------------------
	jmp WORD BOOT_SEG:NewCS

;Put all messages in one place for easy editing
;----------------------------------------------------------------------------------------
msgWelcome	db "Starting JonOS v0.0.1 [Genesis]",13,10,
		db "Press any key...",13,10,0
msgReadError	db "Error reading floppy, retrying...",13,10,0
msgExtendedMem1	db "Found atleast ",0
msgExtendedMem2	db "kb of extended memory.",13,10,0
msgReboot	db "Press r 4 reboot",13,10,0

msgTransferOk	db 13,10,13,10,13,10,13,10,
		db "        Kernel transfered!",13,10,
		db "   Press any key to jump to it!",13,10,
		db "          [R for reboot]",13,10,0


tmpStr		db 0,0,0,0,0,0	;used by the conv routine


;Set all segment registers equal to prevent any strange behaviours

NewCS:	push cs
	push cs
	pop ds
	pop es

;Set up stack from STACK_SEG:STACK_BOTTOM to STACK_SEG:0
	mov ax, STACK_SEG
	mov ss, ax
	mov sp, STACK_BOTTOM

;Clear screen, display welcome msg and wait for key press
	mov si, msgWelcome
	call cls
	call putstr
	call getch

;Show extended memory size <= 64MB through BIOS int 0x88, func 0x15
	mov ah, 0x88
	int 0x15
	mov si, msgExtendedMem1
	call putstr
	call conv
	mov si, msgExtendedMem2
	call putstr

;Transfer SECTORS to CODE_SEG, starting from sector 2
;to location CODE_SEG:0 and continue execution from there
;This is basically "transfer kernel to memory"
;----------------------------------------------------------------------------------------

TransferKernel:
	mov dl, 0x0		;drive = 0 - floppy
	mov ax, 0x0		;reset disk system
	int 0x13
	jc TransferKernel	;failed? try again..
	mov bx, CODE_SEG	;segment to put code in
	mov es, bx		;moves code to es:bx
	xor bx, bx		;offset 0 of segment CODE_SEG
	mov ah, 0x2		;read sector function
	mov al, SECTORS		;# of sectors to transfer
	mov ch, 0x0		;cylinder
	mov cl, 0x2		;sector 1=boot sector (this sector), sector 2+ = JonOS
	mov dh, 0x0		;head
	mov dl, 0x0		;drive 0 - floppy
	int 0x13
	jc ReadingError		;failed? try again...
	jmp TransferOk		;no failure? good!
ReadingError:
	mov si, msgReadError	;set for bad news
	call putstr		;and show them
	jmp TransferKernel	;try again...

;Ok, now the kernel is in place, simply jump to it and it should
;take care of itself from there
;----------------------------------------------------------------------------------------

TransferOk:
	mov ax, 0x13		;set display in 320x200x8 mode
	int 0x10
	mov si, msgTransferOk	;display good news in big format
	call putstr
	mov ax, 0x1010		;set DAC color register
	mov bx, 0x7		;color register to set
	mov ch, 0x10		;green value
	mov cl, 0x20		;red value
	mov dh, 0x30		;blue value
	int 0x10		;flip the colors
	call getch		;wait for keypress and see if
	cmp al, "r"		;user decides to reboot
	je reboot

;If we have a go ahead, launch up JonOS!
;----------------------------------------------------------------------------------------

;	jmp WORD CODE_SEG:0	;another way of doing the same as below
	push word CODE_SEG
	push word 0x0
	retf

;----------------------------------------------------------------------------------------
;Put string
;si = address of string to display
;----------------------------------------------------------------------------------------
putstr:	pusha
	cld		;direction forward
strlp:	lodsb
	or al, al
	jz putstr_exit
	mov ah, 0xe
	mov bx, 0x7
	int 0x10
	jmp strlp
putstr_exit:
	popa
	ret

;----------------------------------------------------------------------------------------
;Clear Screen
;Input/Output: None
;----------------------------------------------------------------------------------------
cls:	pusha
	mov ax, 0x3
	int 0x10
	popa
	ret

;----------------------------------------------------------------------------------------
;Get Character
;Input: none
;Output: al - ASCII character in
;Registers destroyed: ax
;----------------------------------------------------------------------------------------
getch:	xor ax, ax
	int 0x16
	ret

;----------------------------------------------------------------------------------------
;Hex2Dec - Convert hex value in
;ax to printed out decimal value
;Input: ax
;Output: Decimal in tmpStr and on screen
;----------------------------------------------------------------------------------------
conv:	pusha
	xor cx, cx
	mov bx, 0xa
dloop:	xor dx, dx
	div bx
	push dx
	inc cx
	cmp ax, 0
	jne dloop
	mov di, tmpStr
ploop:	pop dx
	add dl, 30h
	mov [ds:di], dl
	inc di
	dec cx
	jnz ploop
	mov si, tmpStr
	call putstr
	popa
	ret

;----------------------------------------------------------------------------------------
;Just a reboot, once called there is no return!
;----------------------------------------------------------------------------------------
reboot:
	mov bx, 0x40
	mov ds, bx
	mov bx, 0x72
	mov ax, 0x1234
	mov [bx], ax
	mov al, 0xfe
	mov dx, 0x64
	out dx, al

;----------------------------------------------------------------------------------------
;Make the sector size 512 bytes
;with last word 0x55AA (0101010110101010 binary)
;This signature is recognised by BIOS as a
;valid bootable sector
;----------------------------------------------------------------------------------------

TIMES 510-($-$$) db 0
dw SIGNATURE
