;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;                                                                                        ;
; JonOS [Genesis] 0.0.1                                                                  ;
;  Made by GigaWolf (GigaWolf@Hotmail.com)                                               ;
;   Released under the GNU General Public License, Version 2                             ;
;                                                                                        ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;


%define STACK_BASE	0x3000
%define STACK_TOP	0xFFFF

[BITS 16]
[ORG 0x10000]

;We end up here aster the bootstrap made its kernel jump
;Initialize stack and hook JonOS Service Interrupts
;----------------------------------------------------------------------------------------
	push cs			;make sure cs = ds
	pop ds
	
	cli			;make sure we don't get any interrupts while
	push word STACK_BASE	;changing the stack!
	pop ss
	mov sp, STACK_TOP
	sti			;ok, stack initialized, allow interrupts

;Set typematic rate
;----------------------------------------------------------------------------------------
	mov ah, 3		;set typematic rate
	mov al, 5		;set typematic rate/delay
	mov bh, 0		;0=250ms, 1=500ms, 2=750ms, 3=1000ms
	mov bl, 0		;30.0
	int 0x16

;Here the service routine gets hooked!
;Interrupt vector table starts at 0:0 and ends at 0:400h - that's 1024 bytes.
;Entries in the Interrupt Table are 4 bytes far pointers in OFFSET:SEGMENT low-endian
;format. Address 1234:ABCD is thus stored in memory as "3412CDAB".
;
;They are nothing more than far pointers to where the CPU should start execution
;when INTn occures. CPU pushes all registers on the stack
;(that's why we disabled interrupts while setting the stack up) and then jumps to
;CS:IP for INTn defined in the Interrupt Vector Table (IVT from now on).
;
;Pointer to INT0 is at 0:0, INT1 is at 0:4, INT2 is at 0:8, INTn is at 0:4n and so on.
;1024 / 4 = 256, there are thus 256 possible interrupts.
;----------------------------------------------------------------------------------------
	push word 0x0		;set es = 0 because of the stosw instruction while
	pop es			;setting up JonOS Service Interrupt

	mov di, 0x21*4+0	;di = INT 0x21 in IVT
	mov ax, Int0x21		;ax = Int0x21 offset
	stosw			;store ax in es:di
	mov di, 0x21*4+2	;di = INT 0x21 code segment
	mov ax, cs		;INT 0x21 code is in this segment..
	stosw

	mov di, 0x1C*4+0
	mov ax, Int0x1C
	stosw
	mov di, 0x1C*4+2
	mov ax, cs
	stosw

;============= Initialize COM1 Offsets and stuff.. ======================================
	push ax
	push word 0x40		;get COM1 address into UART_BASEADDR
	pop es
	xor bx, bx
	mov ax, [es:0]
	mov [COM1_UART_BASEADDR], ax	;UART Base Address
	inc ax				;IER - Interrupt Enable Register
	mov [COM1_UART_IER], ax
	inc ax
	inc ax				;LCR - Line Control Register
	mov [COM1_UART_LCR], ax
	inc ax				;MCR - Modem Control Register
	mov [COM1_UART_MCR], ax
	inc ax				;LSR - Line Status Register
	mov [COM1_UART_LSR], ax
	inc ax				;MSR - Modem Status Register
	mov [COM1_UART_MSR], ax
	pop ax
;----------------------------------------------------------------------------------------


	push cs			;set cs = es to avoid further conflicts
	pop es

;----------------------------------------------------------------------------------------
;Well, here is the main kernel
;
; cs = ds = es = 0x1000
; ss =           0x3000
;----------------------------------------------------------------------------------------

kernel:
	call Init_COM1
	call set_video_mode

	mov si, msgLoaded1	;display a welcome message
	mov bx, 0x9		;in blue
	call put_color_string

	call detect_mem
	call cmd1

	mov si, msgLoaded2	;and another one
	mov bx, 0x9		;in blue
	call put_color_string

	call prompt

main_loop:
	call get_command	;this is the command prompt
	jmp main_loop

;========================================================================================
;========================================================================================
;========================================================================================

;Interrupts go here..
;I found a weird behaviour in the programmable int 0x1C -- it turns out that
;register ds becomes 0x0040 and es becomes 0x1000 (or sometimes 0xA000?)
;I've searched for an explanation without finding one.. I gave up - only solution
;is to initialize all segments to the same value of cs manually!
;x86 architecture really sucks! Strange behaviours, unneccesarily hard, strange
;conventions, f*cked up ways of doing things.. all you can think of!
;I wonder if pmode is the same..
;
;I'm back - now the time is Pi (03:14), and i found what i was searching for..
;Basically i can count on that cs and ss:sp is correct - the rest can be anything
;and that pretty much explains this shitty situation.. take a look at this:
;http://www.faqs.org/faqs/msdos-programmer-faq/part4/section-22.html
;----------------------------------------------------------------------------------------

Int0x21:			;here goes the Int 0x21, i havnt decided what
	iret			;to do with it yet though

Int0x1C:			;havnt decided what to do with it yet
	iret


;COM1 IRQ Service ----- UNFINNISHED!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
;----------------------------------------------------------------------------------------
;Because of lack of documentation and time, i'll have to code this very
;unoptimized. I don't know how interrupts work to 100%, so i'm doing unneccesary
;things at many places, for ex. i check if we have a char in RSR or RBR? even though
;an interrupt triggered this service. I just don't know if the interrupt got triggered
;when the byte got into the buffer or as soon as the first bits started to get in
;and unfortunately i havn't got time to figure that out :( ah well, ill fix it later!
;
;This service reads one byte at a time untill it gets a null-char, and stores everything
;at a specially designated buffer. One byte at a time because i'm afraid of trouble with
;missing int 8's. When it gets a null char, it informs the user a message has arrived :)
;----------------------------------------------------------------------------------------
COM1_IRQ:
	pusha
	mov dx, [COM1_UART_LSR]

COM1_IRQ_GetChar:
	in al, dx
	test al, 1			;see if there's a char in buffer (neccesary?)
	jz COM1_IRQ_GetChar		;we shouldnt end up looping for too long i hope
	and al, 00011110b		;check if we got any errors during transfer
	jnz COM1_IRQ_ReceiveError	;if so, request a new char

	mov dx, [COM1_UART_BASEADDR]	;ok, no errors, fetch the char!
	in al, dx			;into al
	stosb				;and store it in the buffer

	jmp COM1_IRQ_End		;and exit

COM1_IRQ_ReceiveError:
COM1_IRQ_End:
	mov al, 0x20	;tell the PIC we are done servicing this interrupt for now
	out 0x20, al
	popa
	iret
;----------------------------------------------------------------------------------------

;Various messages...
;----------------------------------------------------------------------------------------
msgLoaded1
db "ͻ",13,10,
db " Welcome to JonOS  v0.0.1 [Genesis]                                        ",13,10,
db "                                                                           ",13,10,
db "  A 16-bit operating system made in 100% Assembly, released under the GNU  ",13,10,
db "  This OS comes with absolutely NO WARRANTY, please read the README file!  ",13,10,
db "ͼ",13,10,0

msgLoaded2	db 13,10,"Type 'help' to get started!",13,10,0

%include 'proc.asm'
