;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;                                                                                        ;
; JonOS [Genesis] 0.0.1                                                                  ;
;  Made by GigaWolf (GigaWolf@Hotmail.com)                                               ;
;   Released under the GNU General Public License, Version 2                             ;
;                                                                                        ;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

bits 16
[ORG 0x10000]

%include 'commands.asm'

;Buffers
;----------------------------------------------------------------------------------------

BaseTable			db "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ"

InStreamPointer	dw 0
InStreamBuffer		times 100	db 0	;command buffer
ChatStreamBuffer	times 76	db 0	;serial chat message buffer
;----------------------------------------------------------------------------------------

;--- Color table -------------------------------------------------
;0 = BLACK	1 = BLUE	  2 = GREEN	  3 = CYAN	 |
;4 = RED	5 = MAGENTA	  6 = BROWN	  7 = LIGHTGREY	 |
;8 = DARKGREY	9 = LIGHTBLUE	  A = LIGHTGREEN  B = LIGHTCYAN	 |
;C = LIGHTRED	D = LIGHTMAGENTA  E = YELLOW	  F = WHITE	 |
;-----------------------------------------------------------------

;::::GLOBAL VARIABLES::::
;----------------------------------------------------------------------------------------

VideoMode	db 0x12		;color text
;VideoMode	db 0x7		;b/w text
VideoPage	db 0x0
TextColor	db 0xA
NumberColor	db 0xF
;----------------------------------------------------------------------------------------


;::::GLOBAL STATIC::::
;----------------------------------------------------------------------------------------

newLine	db 13,10,0
testMsg	db "im here!",0
Prompt	db 13,10,">",0

;COMMANDS
;COMMAND_STRING_OFFSETS contain a table of 2-byte pointers to the offsets of the internal
;JonOS commands. COMMAND_OFFSETS contain a similar table to the actual commands.
;COMMAND_STRING_OFFSETS_END is only a bogus value. This is a 2 byte loss -- dirty!
;I could have made it more "clean" - this will be fixed sometime in the future.. maybe :)
;----------------------------------------------------------------------------------------

COMMAND_STRING_OFFSETS		dw cmdString1, cmdString2, cmdString3,
				dw cmdString4, cmdString5, cmdString6, cmdString7,
				dw cmdString8, cmdString9, cmdString10, cmdString11,
				dw cmdString12, cmdString13, cmdString14, cmdString15,
				dw cmdString16, cmdString17, cmdString18, cmdString19,
				dw cmdString20, cmdString21, cmdString22, cmdString23
COMMAND_STRING_OFFSETS_END	dw 0xFFFF

COMMAND_OFFSETS			dw cmd1, cmd2, cmd3, cmd4, cmd5, cmd6, cmd7,
				dw cmd8, cmd9, cmd10, cmd11, cmd12, cmd13, cmd14, cmd15
				dw cmd16, cmd17, cmd18, cmd19, cmd20, cmd21, cmd22, cmd23

cmdString1	db "regdump",0
cmdString2	db "time",0
cmdString3	db "ver",0
cmdString4	db "version",0
cmdString5	db "help",0
cmdString6	db "?",0
cmdString7	db "restart",0
cmdString8	db "reboot",0
cmdString9	db "com1 status",0
cmdString10	db "test",0
cmdString11	db "",0
cmdString12	db "cls",0
cmdString13	db "comchat",0
cmdString14	db "",0
cmdString15	db "",0
cmdString16	db "",0
cmdString17	db "",0
cmdString18	db "",0
cmdString19	db "",0
cmdString20	db "",0
cmdString21	db "",0
cmdString22	db "set color",0
cmdString23	db "set video",0


;----------------------------------------------------------------------------------------
;----------------------------------------------------------------------------------------


;MESSAGES
;----------------------------------------------------------------------------------------



;========================================================================================
;Main Prompt
;========================================================================================

prompt:
	pusha
	mov si, Prompt
	call put_string
	popa
	ret

;----------------------------------------------------------------------------------------
;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
;----------------------------------------------------------------------------------------

;========================================================================================
;Get Command -- Very non optimized...
;========================================================================================
get_command:
	pusha
	push cs				;cs=es to avoid conflicts with some instructions
	pop es
	mov bx, InStreamBuffer		;bx = address of the input stream buffer
	mov di, [InStreamPointer]	;di = how many chars written

	call getch			;fetch next char
	cmp al, 0xd			;see if it is the Enter key
	je execute_command		;if so, do something!
	cmp al, 0x8			;see if it is the Backspace key
	jne not_backspace		;if not, jump over the backspaceing

					;ok, we are in backspace
	cmp di, 0			;see to it that we don't backspace over
	je get_command_exit		;the prompt!

	call put_char			;if everything is ok, do a backspace by
	dec di				;"printing" it - BIOS does the rest, and
	mov [InStreamPointer], di	;decrement+save the "char written" pointer

	push word 0			;also do a backspace in the buffer by setting
	pop word [bx+di]		;a 0 in the last chars place, indicating an
					;end of string in the buffer!
					;otherwise you could type something, backspace
					;it and it would still "be there" in memory!

	mov ah, 9			;BIOS only moves the cursor backwards, we
	mov al, " "			;must manually place a space " " in its place
	mov bx, 7			;without moving the cursor
	mov cx, 1
	int 0x10

	jmp get_command_exit		;now we are done with the backspace, exit..

not_backspace:				;so it wasnt a backspace?
	cmp al, 0x20			;see if it's a normal char in the ASCII table
	jb get_command_exit		;0x20 = Space
	cmp al, 0x7e			;0x7e = ~ and 0x7F is the DEL char, don't use it!
	ja get_command_exit
	cmp di, 78			;ok, char is in a good range but do we have
	je get_command_exit		;enugh space in the buffer to write it?

	call put_char			;yes we do, so first show the char
	mov [bx+di], al			;then put it in the buffer
	inc di				;update the pointer
	mov [InStreamPointer], di	;and save it

get_command_exit:			;we are done, or something went wrong
	popa				;clean up and
	ret				;exit

execute_command:			;ok, user pressed enter, do something!
	cmp di, 0			;but first see if user wrote something to do...
	je get_command_exit		;if not, just exit
	mov bx, COMMAND_STRING_OFFSETS	;otherwise set up bx to point to the string of
	jmp recognise_command		;internal JonOS commands and tro to understand
					;what was written by comparing string with all
					;internal commands.. there is no advanced
					;parameter passing, just char by char check of
					;each command

recognised_command:			;we end up here if we have a recognised command!
	sub bx, COMMAND_STRING_OFFSETS	;first set up bx to point to the offset of the
	add bx, COMMAND_OFFSETS		;command that is going to be executed
	call reset_InStreamBuffer	;clear the input buffer
	call [bx]			;and call the command by its address, pointed to
					;by bx -- this is a very critical part!

	call prompt			;hopefully, command executed and we end up here
	jmp get_command_exit		;so show a new prompt and exit

recognise_command:			;here we try to recognise user input by comparing
	call compare_strings		;input stream with internal commands
	je recognised_command		;if command was recognised, good!

not_recognised:					;if not, let bx point to the offset of
	add bx, 2				;the next internal command string and
	cmp bx, COMMAND_STRING_OFFSETS_END	;ofcourse check if there is a next one!
	jne recognise_command			;if we still have command strings left to
						;check, run the 'compare string' thing
						;again

unknown_command:			;we end up here if we compared the input buffer
	call reset_InStreamBuffer	;with all internal JonOS commands without a
	mov si, msgUnknownCommand	;single match. so let the user know it
	call put_string			;by displaying a message,
	call prompt			;showing a new prompt to make him try again
	jmp get_command_exit		;and we are finnished!


reset_InStreamBuffer:			;zero out whole input stream buffer regardless of
	mov di, InStreamBuffer		;where the imput stream pointer is - thats not
	xor ax, ax			;important afterall
	mov cx, 80			;make it above the limit just in case - buffer is
	repnz stosb			;larger than that afterall.. and store 0000000...
	mov [InStreamPointer], cx	;after storing, cx is 0 because of repnz, so
	ret				;store it (0) as the new pointer and exit


compare_strings:			;here we compare strings, first make sure that
	cld				;si and di are incremented in the right way
	mov cx, [InStreamPointer]	;and only compare cx bytes (as much as the user
	mov si, InStreamBuffer		;wrote). SI points at the imput buffer and
	mov di, [bx]			;di points to an internal JonOS command string
	repe cmpsb			;defined and updated in execute/recognise command
	jne compare_strings_failed	;respectively.. if something turned out not to
					;match, let the user know..

	cmpsb				;but if every char input matched so far, check if
	jne compare_strings_failed	;the next byte of input stream and internal JonOS
	ret				;command is 0, in order to ensure that is was the
					;whole command. otherwise 'vers' and 'version'
					;would be treated as the same - and we don't want
					;that!

compare_strings_failed:			;ok, so the strings did not match
	ret				;just exit

msgUnknownCommand	db 13,10,"Unknown Command",13,10,0

;----------------------------------------------------------------------------------------
;=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
;----------------------------------------------------------------------------------------

set_video_mode:
	pusha
	mov al, [VideoMode]
	xor ah, ah
	int 0x10
	popa
	ret


;========================================================================================
;Put string
;si = address of string to display
;========================================================================================
put_string:
	pusha
	cld
put_string_loop:
	lodsb
	or al, al
	jz put_string_exit
	mov ah, 0xe
	mov bh, [VideoPage]
	mov bl, [TextColor]
	int 0x10
	jmp put_string_loop
put_string_exit:
	popa
	ret


;========================================================================================
;Put Color String
;si = address of string do display
;bl = color of string
;========================================================================================

put_color_string:
	pusha
	cld
put_color_string_loop:
	lodsb
	or al, al
	jz put_color_string_exit
	mov ah, 0xe
	mov bh, 0
	int 0x10
	jmp put_color_string_loop
put_color_string_exit:
	popa
	ret

;========================================================================================
;Put Number
;Takes number color from the
;NumberColor variable
;al = number
;========================================================================================
put_number:
	pusha
	mov ah, 0xe
	mov bh, [VideoPage]
	mov bl, [NumberColor]
	int 0x10
	popa
	ret

;========================================================================================
;al = char to display
;========================================================================================
put_char:
	pusha
	mov ah, 0xe
	mov bh, [VideoPage]
	mov bl, [TextColor]
	int 0x10
	popa
	ret
;========================================================================================
;al = char to display, bl = color
;========================================================================================
put_color_char:
	pusha
	mov ah, 0xe
	mov dh, [VideoPage]
	int 0x10
	popa
	ret

;========================================================================================
;Get Character
;Input: none
;Output: al - ASCII character in
;Registers destroyed: ax
;========================================================================================
getch:	xor ax, ax
	int 0x16
	ret


;========================================================================================
;Clear Screen
;Input/Output: None
;========================================================================================
cls:	pusha
	xor ah, ah
	mov al, [VideoMode]
	int 0x10
	popa
	ret


;========================================================================================
;Detect extended memory size <= 64 MB
;========================================================================================

msgExtendedMem1	db "Found atleast ",0
msgExtendedMem2	db " kb of extended memory.",0

detect_mem:
	pusha
	mov ah, 0x88			;extended memory size determination
	int 0x15
	mov bx, 0xa			;base 10
	mov si, msgExtendedMem1
	call put_string
	call convert
	mov si, msgExtendedMem2
	call put_string
	popa
	ret


;========================================================================================
;Base convert - converts numbers to other bases.
;Prints result directly on screen.
;
;Input:	ax - unsigned number
;	bx - base
;
;Destroyed registers: None
;========================================================================================

convert:
	pusha
	xor cx, cx
	mov si, BaseTable
convert_loop1:
	xor dx, dx
	div bx
	push dx
	inc cx
	cmp ax, 0
	jne convert_loop1
convert_loop2:
	pop bx
	mov al, [bx+si]
	call put_number
	dec cx
	jnz convert_loop2
	popa
	ret
;----------------------------------------------------------------------------------------


;========================================================================================
;Initialize COM1 @ 0x3F8, No registers destroyed,  Input/Output: NONE
;========================================================================================

Init_COM1:
	pusha
	mov dx, [COM1_UART_LCR]		;LCR - Line Control Register
	mov al, 0			;Clear DLAB (Divisor Latch Access Bit)
	out dx, al
	mov dx, [COM1_UART_IER]		;IER - Interrupt Enable Register
	mov al, 0			;Turn off all interrupts
	out dx, al
	mov dx, [COM1_UART_LCR]		;LCR - Line Control Register
	mov al, 0x80			;Set DLAB (Divisor Latch Access Bit)
	out dx, al
	mov dx, [COM1_UART_BASEADDR]	;Divisor
	mov ax, [COM1_UART_BAUDRATE]	;COM1 speed
	out dx, ax
	mov dx, [COM1_UART_LCR]		;LCR - Line Control Register
	mov al, [COM1_UART_LCRVAL]	;Set parameters
	out dx, al
	mov dx, [COM1_UART_MCR]		;MCR
	xor ax, ax			;Clear loopback
;	mov al, 00000001b		;Experimental!!! - delete if fails!!
	out dx, al
	popa
	ret

;========================================================================================
;Theese values get initialized at kernel boot, so they can vary at runtime.. don't
;count on theese hardcoded ones!  But they should be accurate for COM1 atleast.. hmm
;========================================================================================
COM1_UART_BASEADDR	dw 0x3F8	;COM1
COM1_UART_IER		dw 0x3F9	;Serial Interupt Enable Register, @ComBase+1
COM1_UART_LCR		dw 0x3FB	;Line Control Register, @ComBase+3
COM1_UART_MCR		dw 0x3FC	;Modem Control Register, @ComBase+4
COM1_UART_LSR		dw 0x3FD	;Line Status Register, @ComBase+5
COM1_UART_MSR		dw 0x3FE	;Modem Status Register, @ComBase+6
COM1_UART_BAUDRATE	dw 0x2		;Divisor: 2 -> 57600 baud works OK on my 386DX33
COM1_UART_LCRVAL	db 0x1b		;0001 0011 for 8n1
COM1_UART_FCRVAL	db 0x87		;?? :o :O

COM2_UART_BASEADDR	dw 0
COM2_UART_IER		dw 0
COM2_UART_LCR		dw 0
COM2_UART_MCR		dw 0
COM2_UART_LSR		dw 0
COM2_UART_MSR		dw 0
COM2_UART_BAUDRATE	dw 0
COM2_UART_LCRVAL	db 0
COM2_UART_FCRVAL	db 0

COM3_UART_BASEADDR	dw 0
COM3_UART_IER		dw 0
COM3_UART_LCR		dw 0
COM3_UART_MCR		dw 0
COM3_UART_LSR		dw 0
COM3_UART_MSR		dw 0
COM3_UART_BAUDRATE	dw 0
COM3_UART_LCRVAL	db 0
COM3_UART_FCRVAL	db 0

COM4_UART_BASEADDR	dw 0
COM4_UART_IER		dw 0
COM4_UART_LCR		dw 0
COM4_UART_MCR		dw 0
COM4_UART_LSR		dw 0
COM4_UART_MSR		dw 0
COM4_UART_BAUDRATE	dw 0
COM4_UART_LCRVAL	db 0
COM4_UART_FCRVAL	db 0

;----------------------------------------------------------------------------------------

;========================================================================================
;Print Line Status Register of COM1 in human readable form by register polling (no BIOS)
;========================================================================================

Print_COM1_Status:
	pusha
	mov si, newLine
	call put_string
	mov dx, [COM1_UART_LSR]		;Serial Line Status Register
	in al, dx

Probe_COM1UART_LSR_bit_0:			;Received Data Available (RDA) ?
	test al, 00000001b
	jz Probe_COM1UART_LSR_bit_1
	mov si, DataReady
	call put_string
Probe_COM1UART_LSR_bit_1:			;Overrun Error?
	test al, 00000010b
	jz Probe_COM1UART_LSR_bit_2
	mov si, OverrunError
	call put_string
Probe_COM1UART_LSR_bit_2:			;Parity Error?
	test al, 00000100b
	jz Probe_COM1UART_LSR_bit_3
	mov si, ParityError
	call put_string
Probe_COM1UART_LSR_bit_3:			;Framing Error?
	test al, 00001000b
	jz Probe_COM1UART_LSR_bit_4
	mov si, FramingError
	call put_string
Probe_COM1UART_LSR_bit_4:			;Break Detected?
	test al, 00010000b
	jz Probe_COM1UART_LSR_bit_5
	mov si, BreakDetect
	call put_string
Probe_COM1UART_LSR_bit_5:			;THR (Transmit Holding Register) Empty?
	test al, 00100000b
	jz Probe_COM1UART_LSR_bit_6
	mov si, TransmitHoldingRegisterEmpty
	call put_string
Probe_COM1UART_LSR_bit_6:			;TSR (Transmit Shift Register) Empty?
	test al, 01000000b
	jz Probe_COM1UART_LSR_bit_7
	mov si, TransmitShiftRegisterEmpty
	call put_string
Probe_COM1UART_LSR_bit_7:			;Time Out Error?
	test al, 10000000b			;(N/A for BIOS functions 1&2 ??)
	jz Probe_COM1UART_LSR_bit_End
	mov si, TimeOut
	call put_string

Probe_COM1UART_LSR_bit_End:
	popa
	ret

DataReady	db "Data Ready!",13,10,0					;bit 0
OverrunError	db "Overrun Error - Data has been lost!",13,10,0		;bit 1
ParityError	db "Parity Error!",13,10,0					;bit 2
FramingError	db "Framing Error!",13,10,0					;bit 3
BreakDetect	db "Break - Broken Line Detected!",13,10,0			;bit 4
TransmitHoldingRegisterEmpty	db "Transmit Holding Register Empty!",13,10,0	;bit 5
TransmitShiftRegisterEmpty	db "Transmit Shift Register Empty!",13,10,0	;bit 6
TimeOut		db "Timeout Error!",13,10,0					;bit 7

Print_COM1_BIOS_Status:
	pusha
	mov si, newLine
	call put_string

	mov ah, 3
	xor dx, dx	;COM1
	int 0x14
	xchg ah, al
	jmp Probe_COM1UART_LSR_bit_0
;----------------------------------------------------------------------------------------

;========================================================================================
;Read byte from COM1
;LSR in AH & Character in AL on return
;========================================================================================
Get_COM1
	push dx		;we will use dx as port pointer
	cld		;we are not arabes, we write to the buffer the right way! =)
	mov dx, [COM1_UART_LSR]		;fetch the LSR
	in al, dx			;to al
	mov ah, al			;save the LSR in ah
	test ah, 1			;see if we have a new byte in the buffer
	jz Get_COM1_Exit		;if not, go back to the callee
	mov dx, [COM1_UART_BASEADDR]	;we come here only if there is a char in buffer
	in al, dx			;get the char and return to callee
Get_COM1_Exit
	pop dx
	ret
;----------------------------------------------------------------------------------------


;========================================================================================
;Send byte in AL to COM1, No registers destroyed..
;========================================================================================
Send_COM1
	push dx		;we will use dx as port pointer
	cld		;we are not arabes, we write to the buffer the right way! =)
	push ax				;save the byte to be sent
	mov dx, [COM1_UART_LSR]		;fetch the LSR
Send_COM1_loop:
	in al, dx			;to al
	test al, 100000b		;see if THR is empty so we can send a char
	jz Send_COM1_loop		;no? baudrate decides how long till it will be..
	mov dx, [COM1_UART_BASEADDR]
	pop ax				;ok, now we can send it so pop it down
	out dx, al			;and there it goes..
	pop dx				;clean up
	ret
;----------------------------------------------------------------------------------------

;========================================================================================
;Send NULL-terminated string pointed to by ES:SI through COM1
;========================================================================================
Send_COM1_String:
	pusha
	popa
	ret
;----------------------------------------------------------------------------------------

;========================================================================================
;Wait for and get NULL-terminated string from COM1 and put it at ES:DI
;========================================================================================
Get_COM1_String:
	pusha
	popa
	ret
;----------------------------------------------------------------------------------------
